/* -*- linux-c -*- */
/**
 * ビット列に変換
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include "../lib/signal_spec.h"

/**
 * ビット列変換メイン
 *
 * @param[in]  stream_in  入力ストリーム
 * @param[out] stream_out 出力ストリーム
 */
void bitstream_main ( FILE *stream_in, FILE *stream_out, int fs )
{
	int s_in;
	int t_dur_sample = 0;
	int s_dur = 999;

	int t_bps =  fs / BPS + 0.5;  /**< １符号のサンプル数 */
	int t_bps2 =  fs / (2*BPS) + 0.5;  /**< 半符号のサンプル数 */

	while ( !feof ( stream_in ) ) {
		s_in = fgetc ( stream_in );
		if ( s_in < 0 )
			break;

		if ( s_dur != s_in ) {
			/* 変化点検出 */
			t_dur_sample = t_bps2;
			s_dur = s_in;
		} else if ( t_bps < t_dur_sample ) {
			/* 符号検出 */
			t_dur_sample -= t_bps;
			fputc ( s_dur, stream_out );
			fflush ( stream_out );
		}
		++t_dur_sample;
	}
}



/**
 * メインルーチン
 *
 * 第一引数はサンプリング周波数．単位は Hz（省略値は 8000）
 *
 * @param[in] argc 引数の個数
 * @param[in] argv 引数
 */
int main ( int argc, char *argv[] )
{
	int fs = Fs;

	if ( argc >= 2 ) {
		fs = atoi ( argv[1] );
	}

	bitstream_main ( stdin, stdout, fs );
	return 0;
}
