/* -*- linux-c -*- */
/**
 * ビット列に変換
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 * 
 */
#include <stdio.h>
#include "../lib/signal_spec.h"
#include "../lib/io.h"


/**
 * ビット列変換メイン
 *
 * @param[in]  stream_in  入力ストリーム
 * @param[out] stream_out 出力ストリーム
 */
void bitstream_main ( FILE *stream_in, FILE *stream_out )
{
	sample_t s_in;
	double t_dur_sample = 0;
	int a_dur = NUL;
	int s_in_a_int = NONE;

	while ( !feof ( stream_in ) ) {
		if ( sample_read ( &s_in, 1, stream_in ) < 1 ) {
			break;
		}
		if ( s_in.a > 0.5 ) {
			s_in_a_int = MARK;
		} else if ( s_in.a < -0.5 ) {
			s_in_a_int = SPACE;
		} else {
			s_in_a_int = NONE;
		}

		if ( a_dur != s_in_a_int ) {
			/* 変化点検出 */
			t_dur_sample = s_in.t + T_BPS/2;
			a_dur = s_in_a_int;
		} else if ( t_dur_sample < s_in.t ) {
			/* 符号検出 */
			t_dur_sample += T_BPS;
			fputc ( a_dur, stream_out );
			fflush ( stream_out );
		}
	}
}


/**
 * メインルーチン
 */
int main ( int argc, char *argv[] )
{
	bitstream_main ( stdin, stdout );
	return 0;
}
