/* -*- linux-c -*- */
/**
 * チャタリング除去
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 */
#include <stdio.h>
#include <stdlib.h>
#include "../lib/signal_spec.h"

#define T_OUT ((int)(Fs/(5*BPS)+0.5)) /**< 符号変化と判定するまでのサンプル数 */

/**
 * チャタリング除去メイン
 *
 * @param[in]  stream_in  入力ストリーム
 * @param[out] stream_out 出力ストリーム
 */
void dechatter_main ( FILE *stream_in, FILE *stream_out, int fs )
{
	int s_in = NONE;
	int s_prev = NONE;
	int s_dur = NONE;
	int t_dur = 0;

	int t_out = fs/(5*BPS) + 0.5; /**< 符号変化と判定するまでのサンプル数 */
	while ( !feof ( stream_in ) ) {
		s_in = fgetc ( stream_in );
		if ( s_in < 0 )
			break;

		if ( s_in != s_dur ) {
			/* 符号変化を検出 */
			s_dur = s_in;
			t_dur = 0;
		} else if ( t_dur > t_out ) {
			/* 符号変化観察時間完了 */
			s_prev = s_dur;
		} else {
			/* s_in == s_dur */
			++t_dur;
		}
		fputc ( s_prev, stream_out );
		//fputc ( '\n', stream_out );
		fflush ( stream_out );
	}
}



/**
 * メインルーチン
 */
int main ( int argc, char *argv[] )
{
	int fs = Fs;

	if ( argc >= 2 ) {
		fs = atoi ( argv[1] );
	}

	dechatter_main ( stdin, stdout, fs );
	return 0;
}
