/* -*- linux-c -*- */
/**
 * チャタリング除去
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 * 
 */
#include <stdio.h>
#include "../lib/io.h"
#include "../lib/signal_spec.h"

#define T_OUT (1.0/(5*BPS)) /**< 符号変化と判定するまでの秒数 [sec] */

/**
 * チャタリング除去メイン
 *
 * @param[in]  stream_in  入力ストリーム
 * @param[out] stream_out 出力ストリーム
 */
void dechatter_main ( FILE *stream_in, FILE *stream_out )
{
	sample_t s_in, s_out;
	int a_prev = A_NONE;
	int a_dur = A_NONE;
	int s_in_a_int = A_NONE;
	double t_dur_start = 1e60;

	while ( !feof ( stream_in ) ) {
		if ( sample_read ( &s_in, 1, stream_in ) < 1 ) {
			break;
		}
		if ( s_in.a > 0.5 ) {
			s_in_a_int = A_MARK;
		} else if ( s_in.a < -0.5 ) {
			s_in_a_int = A_SPACE;
		} else {
			s_in_a_int = A_NONE;
		}

		if ( s_in_a_int != a_dur ) {
			/* 符号変化を検出 */
			a_dur = s_in_a_int;
			t_dur_start = s_in.t;
		} else if ( s_in.t - t_dur_start > T_OUT ) {
			/* 符号変化観察時間完了 */
			a_prev = a_dur;
			t_dur_start = 1e60;
		}
		s_out.t = s_in.t;
		s_out.a = a_prev;
		sample_write ( &s_out, 1, stream_out );
		fflush ( stream_out );
	}
}



int main ( int argc, char *argv[] )
{
	dechatter_main ( stdin, stdout );
	return 0;
}
