/* -*- linux-c -*- */
/**
 * ビット列の復号
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 */
#include <stdio.h>
#include <stdlib.h>
#include "../lib/codetable.h"
#define NMEMB(ary) (sizeof(ary)/sizeof(ary[0]))

#if 0
#define debug(form,args...) fprintf(stderr,"%d:"form,__LINE__,##args)
#else
#define debug(form,args...)
#endif


/* 地域ビットテーブル */
typedef struct {
	char *name;
	unsigned bits;
} chiiki_bits_t;
chiiki_bits_t *chiiki_table;

/* 日ビットテーブル */
typedef struct {
	int day;
	unsigned bits;
} hi_bits_t;
hi_bits_t *hi_table;

/* 月ビットテーブル */
typedef struct {
	int month;
	unsigned bits;
} tsuki_bits_t;
tsuki_bits_t *tsuki_table;

/* 時ビットテーブル */
typedef struct {
	int hour;
	unsigned bits;
} zi_bits_t;
zi_bits_t *zi_table;

/* 年ビットテーブル */
typedef struct {
	int year;
	unsigned bits;
} nen_bits_t;
nen_bits_t *nen_table;


/* １ロングワードめ */
/* 前置符号 */
unsigned bits_zenchi_12, bits_zenchi_end;
unsigned mask_zenchi = 0x0000000f;

/* 固定符号 */
unsigned bits_kotei_1end, bits_kotei_2;
unsigned mask_kotei = 0xffff0000;
/* 地域区分符号の開始部 */
unsigned bits_chiiki_pre_12, bits_chiiki_pre_end;
unsigned mask_chiiki_pre = 0x0000c000;
/* 地域区分符号本体 */
unsigned mask_chiiki = 0x000003ffc;
/* 地域区分符号の末尾部 */
unsigned bits_chiiki_post_12, bits_chiiki_post_end;
unsigned mask_chiiki_post = 0x00000003;

/* 月日区分符号 */
unsigned bits_tsukihi_pre_12, bits_tsukihi_pre_end;
unsigned mask_tsukihi_pre = 0x0000e000;
/* 日 */
unsigned mask_hi = 0x00001f00;
/* 月日区切り */
unsigned mask_tsukihi_mid = 0x00000080;
unsigned bits_tsukihi_mid_odd, bits_tsukihi_mid_even;
/* 月 */
unsigned mask_tsuki = 0x0000007c;
/* 月日後置末尾部 */
unsigned bits_tsukihi_post_12, bits_tsukihi_post_end;
unsigned mask_tsukihi_post = 0x00000003;

/* 年時区分符号 */
unsigned bits_nenzi_pre_12, bits_nenzi_pre_end;
unsigned mask_nenzi_pre = 0x0000e000;
/* 時 */
unsigned mask_zi = 0x00001f00;
/* 年時区切り */
unsigned bits_nenzi_mid_odd, bits_nenzi_mid_even;
unsigned mask_nenzi_mid = 0x00000080;
/* 年 */
unsigned mask_nen = 0x0000007c;
/* 年時符号末尾部 */
unsigned bits_nenzi_post_12, bits_nenzi_post_end;
unsigned mask_nenzi_post = 0x00000003;

/**
 * 符号表ビット列の前処理
 *
 * @param[in] bitseq ビット列．ENDCODE が番人
 * @param[in] shift  シフトするビット数
 */
unsigned pre_compile_bitseq ( char bitseq[], int shift )
{
	unsigned ret = 0;
	int i;
	for ( i = 0;;++i ) {
		if ( bitseq[i] == ENDCODE )
			break;
		ret = (ret << 1) |  bitseq[i];
	}
	return ret<<shift;
}


/**
 * 符号表の前処理
 *
 * ビット演算で比較できるようにする
 */
void pre_compile_tables ( void )
{
	int i, c;

	/* 前置符号 */
	bits_zenchi_12  = pre_compile_bitseq ( code_zenchi_12,  0 );
	bits_zenchi_end = pre_compile_bitseq ( code_zenchi_end, 0 );

	/* 固定符号 */
	bits_kotei_1end = pre_compile_bitseq ( code_kotei_1end, 16 );
	bits_kotei_2    = pre_compile_bitseq ( code_kotei_2,    16 );

	/* 地域符号開始部 */
	bits_chiiki_pre_12  = pre_compile_bitseq ( code_chiiki_pre_12,  14 );
	bits_chiiki_pre_end = pre_compile_bitseq ( code_chiiki_pre_end, 14 );

	/* 地域符号 */
	for ( c = 0; chiiki_codeset[c].name != NULL; ++c )
		;
	chiiki_table = malloc ( (c + 1)*sizeof(chiiki_bits_t) );
	for ( i = 0; i <= c; ++i ) {
		chiiki_table[i].name = chiiki_codeset[i].name;
		chiiki_table[i].bits =
			pre_compile_bitseq ( chiiki_codeset[i].code, 2 );
	}
	
	/* 地域符号末尾部 */
	bits_chiiki_post_12  = pre_compile_bitseq ( code_chiiki_post_12,  0 );
	bits_chiiki_post_end = pre_compile_bitseq ( code_chiiki_post_end, 0 );

	/* 月日区分符号開始部 */
	bits_tsukihi_pre_12  = pre_compile_bitseq ( code_tsukihi_pre_12,  13 );
	bits_tsukihi_pre_end = pre_compile_bitseq ( code_tsukihi_pre_end, 13 );
	
	/* 日 */
	for ( c = 0; hi_codeset[c].day >= 0; ++c )
		;
	hi_table = malloc ( (c + 1)*sizeof(hi_bits_t) );
	for ( i = 0; i <= c; ++i ) {
		hi_table[i].day = hi_codeset[i].day;
		hi_table[i].bits =
			pre_compile_bitseq ( hi_codeset[i].code, 8 );
	}
	/* 月日区切り */
	bits_tsukihi_mid_odd = pre_compile_bitseq ( code_tsukihi_mid_odd, 7 );
	bits_tsukihi_mid_even = pre_compile_bitseq ( code_tsukihi_mid_even, 7 );
	/* 月 */
	for ( c = 0; tsuki_codeset[c].month >= 0; ++c )
		;
	tsuki_table = malloc ( (c + 1)*sizeof(tsuki_bits_t) );
	for ( i = 0; i <= c; ++i ) {
		tsuki_table[i].month = tsuki_codeset[i].month;
		tsuki_table[i].bits =
			pre_compile_bitseq ( tsuki_codeset[i].code, 2 );
	}

	/* 月日区分符号の末尾部 */
	bits_tsukihi_post_12 = pre_compile_bitseq ( code_tsukihi_post_12, 0 );
	bits_tsukihi_post_end = pre_compile_bitseq ( code_tsukihi_post_end, 0 );

	/* 年時区分符号の開始部 */
	bits_nenzi_pre_12 = pre_compile_bitseq ( code_nenzi_pre_12, 13 );
	bits_nenzi_pre_end = pre_compile_bitseq ( code_nenzi_pre_end, 13 );
	/* 時 */
	for ( c = 0; zi_codeset[c].hour >= 0; ++c )
		;
	zi_table = malloc ( (c + 1)*sizeof(zi_bits_t) );
	for ( i = 0; i <= c; ++i ) {
		zi_table[i].hour = zi_codeset[i].hour;
		zi_table[i].bits =
			pre_compile_bitseq ( zi_codeset[i].code, 8 );
	}
	/* 年時区分符号の区切り */
	bits_nenzi_mid_odd = pre_compile_bitseq ( code_nenzi_mid_odd, 7 );
	bits_nenzi_mid_even = pre_compile_bitseq ( code_nenzi_mid_even, 7 );
	/* 年 */
	for ( c = 0; nen_codeset[c].year >= 0; ++c )
		;
	nen_table = malloc ( (c + 1)*sizeof(nen_bits_t) );
	for ( i = 0; i <= c; ++i ) {
		nen_table[i].year = nen_codeset[i].year;
		nen_table[i].bits =
			pre_compile_bitseq ( nen_codeset[i].code, 2 );
	}
	/* 年時区分符号の末尾部 */
	bits_nenzi_post_12 = pre_compile_bitseq ( code_nenzi_post_12, 0 );
	bits_nenzi_post_end = pre_compile_bitseq ( code_nenzi_post_end, 0 );
}

#define KAISHI_1 1
#define KAISHI_2 2
#define SYUURYOU 4
char *sig_table[] = { "不明",
		     "第一種開始信号",
		     "第二種開始信号",
		     "不明",
		     "終了信号",
		     "不明","不明","不明",};
void decoder_main ( FILE *stream_in, FILE *stream_out )
{
	int in_c;
	int b, i;
	unsigned buf[4] = {0,0,0,0};
	unsigned kotei[4] = {0,0,0,0};

	int sig_cat;  /* 信号の種類 */
	int year, month, day, hour;
	int exact_time;
	char *chiiki; /* 地域 */

	
	for (;;) {
		do {
			in_c = fgetc(stream_in);
		} while ( !(in_c == EOF || in_c == '1' ||
			    in_c == '0' || in_c == 'N' ) );
		if ( in_c == EOF ) {
			break;
		}
		if ( in_c == 'N' ) {
			buf[0] = buf[1] = buf[2] = buf[3] = 0;
			continue;
		}
		b = (in_c == '1') ? 1 : 0;
		/* ビットをバッファに格納 */
		buf[0] = (buf[0] << 1) | ((buf[1] >> 31) ? 1 : 0);
		buf[1] = (buf[1] << 1) | ((buf[2] >> 31) ? 1 : 0);
		buf[2] = (buf[2] << 1) | ((buf[3] >> 31) ? 1 : 0);
		buf[3] = (buf[3] << 1) | b;

		/* 解読開始 */

#if 0
		if ( (buf[0] & mask_zenchi) == bits_zenchi_12 ) {
			sig_cat = KAISHI_1 | KAISHI_2;
		} else if ( (buf[0] & mask_zenchi) == bits_zenchi_end ) {
			sig_cat = SYUURYOU;
		} else {
			continue;
		}
#endif
		/* 固定符号を抜き出す */
		for ( i = 1; i < 4; ++i ) {
			kotei[i] = buf[i] & mask_kotei;
		}
		if ( (kotei[1] == kotei[2]) && (kotei[2] == kotei[3]) ) {
			/* nop */
		} else {
			continue;
		}

		if ( kotei[1] == bits_kotei_1end ) {
			sig_cat = KAISHI_1|SYUURYOU;
 		} else if ( kotei[1] == bits_kotei_2 ) {
			sig_cat = KAISHI_2;
		} else {
			continue;
		}
		
		debug ( "buf: %08x %08x %08x %08x\n",
			buf[0], buf[1], buf[2], buf[3] );

		/* 地域区分符号で信号の種類を確定させる */
		if ((buf[1] & mask_chiiki_pre) == bits_chiiki_pre_12 ) {
			sig_cat &= KAISHI_1|KAISHI_2;
		} else if ((buf[1] & mask_chiiki_pre) == bits_chiiki_pre_end ) {
			sig_cat &= SYUURYOU;
		}
		if ( (sig_cat != KAISHI_1) && (sig_cat != KAISHI_2) &&
		     (sig_cat != SYUURYOU) ) 
			continue;

		/* 固定部分のチェック */
		if ( sig_cat & (KAISHI_1|KAISHI_2) ) {
			/* 地域区分符号の開始部 */
			if ((buf[1] & mask_chiiki_pre) != bits_chiiki_pre_12 )
				continue;
			debug("\n");
			/* 地域区分符号の末尾部 */
			if ((buf[1] & mask_chiiki_post) != bits_chiiki_post_12 )
				continue;
			debug("\n");
			/* 月日区分符号の開始部 */
			if ((buf[2] & mask_tsukihi_pre) != bits_tsukihi_pre_12 )
				continue;
			debug("\n");
			/* 月日区分符号の末尾部 */
			if ((buf[2] & mask_tsukihi_post) !=
			    bits_tsukihi_post_12 )
				continue;
			debug("\n");
			/* 年時区分符号の開始部 */
			if ((buf[3] & mask_nenzi_pre) != bits_nenzi_pre_12 )
				continue;
			debug("\n");
			/* 年時区分符号の末尾部 */
			if ((buf[3] & mask_nenzi_post) != bits_nenzi_post_12 )
				continue;
			debug("\n");
		} else if ( sig_cat & SYUURYOU ) {
			/* 地域区分符号の開始部 */
			if ((buf[1] & mask_chiiki_pre) != bits_chiiki_pre_end )
				continue;
			/* 地域区分符号の末尾部 */
			if ((buf[1] & mask_chiiki_post) != bits_chiiki_post_end )
				continue;
			/* 月日区分符号の開始部 */
			if ((buf[2] & mask_tsukihi_pre) != bits_tsukihi_pre_end )
				continue;
			/* 月日区分符号の末尾部 */
			if ((buf[2] & mask_tsukihi_post) !=
			    bits_tsukihi_post_end )
				continue;
			/* 年時区分符号の開始部 */
			if ((buf[3] & mask_nenzi_pre) != bits_nenzi_pre_end )
				continue;
			/* 年時区分符号の末尾部 */
			if ((buf[3] & mask_nenzi_post) != bits_nenzi_post_end )
				continue;
		}

		debug ( "信号区分 = %s\n", sig_table[sig_cat] );

		/* 年の検索 */
		year = -1;
		for ( i =0; nen_table[i].year >= 0; ++i ) {
			if ((buf[3] & mask_nen) == nen_table[i].bits) {
				year = nen_table[i].year;
				break;
			}
		}
		if ( year < 0 )
			continue;
		debug ("年 = 昭和 %d 年\n", year );

		/* 月 */
		month = -1;
		for ( i =0; tsuki_table[i].month >= 0; ++i ) {
			if ((buf[2] & mask_tsuki) == tsuki_table[i].bits) {
				month = tsuki_table[i].month;
				break;
			}
		}
		if ( month < 0 )
			continue;
		debug ("月 = %d 月\n", month );

		/* 日 */
		day = -1;
		for ( i =0; hi_table[i].day >= 0; ++i ) {
			if ((buf[2] & mask_hi) == hi_table[i].bits) {
				day = hi_table[i].day;
				break;
			}
		}
		if ( day < 0 )
			continue;
		debug ("日 = %d 日\n", day );
		
		/* 時 */
		hour = -1;
		for ( i =0; zi_table[i].hour >= 0; ++i ) {
			if ((buf[3] & mask_zi) == zi_table[i].bits) {
				hour = zi_table[i].hour;
				break;
			}
		}
		if ( hour < 0 )
			continue;
		debug ("時 = %d 時\n", hour );

		if ((buf[2] & mask_tsukihi_mid) || (buf[3] & mask_nenzi_mid)){
			exact_time = 0;
			debug("送出日 = 不正確\n");
		} else {
			exact_time = 1;
			debug("送出日 = 正確\n");
		}

		/* 地域区分符号の検索 */
		chiiki = NULL;
		for ( i = 0; chiiki_table[i].name != NULL; ++i ) {
			if ((buf[1] & mask_chiiki) == chiiki_table[i].bits) {
				chiiki = chiiki_table[i].name;
				break;
			}
		}
		if ( chiiki == NULL )
			continue;
		debug ( "地域 = %s\n", chiiki );

		fprintf ( stream_out, "begin:\n" );
		fprintf ( stream_out, "信号区分: %s\n", sig_table[sig_cat] );
		fprintf ( stream_out, "地域: %s\n", chiiki );
		fprintf ( stream_out, "年: %d\n", year );
		fprintf ( stream_out, "月: %d\n", month );
		fprintf ( stream_out, "日: %d\n", day );
		fprintf ( stream_out, "時: %d\n", hour );
		fprintf ( stream_out, "現在日時: %d\n", exact_time );
		fprintf ( stream_out, "end:\n" );
		fflush ( stream_out );
	}
}


int main ( int argc, char *argv[] )
{
	pre_compile_tables();
	decoder_main ( stdin, stdout );
	return 0;
}
