/* -*- linux-c -*- */
/**
 * 検波器
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 */
#include <stdio.h>
#include <stdlib.h>
#include "../lib/signal_spec.h"

/**
 * 検波メイン
 *
 * @param stream_in  入力ストリーム
 * @param stream_out 出力ストリーム
 */
void detect_main ( FILE *stream_in, FILE *stream_out, int fs )
{
	int s_in, s_out;

	int s_prev = 0;
	int t = 0;

	int t_mark_h  = (int)(fs/(F_MARK  - 192.0) + 0.5);
	int t_mark_l  = (int)(fs/(F_MARK  + 192.0) + 0.5); 
	int t_space_h = (int)(fs/(F_SPACE - 192.0) + 0.5);
	int t_space_l = (int)(fs/(F_SPACE + 192.0) + 0.5);

	s_out = NONE;
	while ( !feof ( stream_in ) ) {
		s_in = fgetc ( stream_in );
		if ( s_in < 0 )
			break;

		s_in -= 128; /* 符号付きに変換 */

		if ( ( s_prev <= 0 ) && ( s_in > 0 ) ) {
			/* マイナス -> プラスのゼロクロス点を検出した */
			if ( (t_mark_l <= t) && (t <= t_mark_h) ) {
				s_out = MARK;
			} else if ( (t_space_l <= t) && (t <= t_space_h) ) {
				s_out = SPACE;
			} else {
				s_out = NONE;
			}
			t = 0;
		} else if ( t_space_h < t ) {
			s_out = NONE;
		}
		s_prev = s_in;

		fputc ( s_out, stream_out );
		//fputc ( '\n', stream_out );
		fflush ( stream_out );
		++t;
	}
}



int main ( int argc, char *argv[] )
{
	int fs = Fs;

	if ( argc >= 2 ) {
		fs = atoi ( argv[1] );
	}

	detect_main ( stdin, stdout, fs );
	return 0;
}
