/* -*- linux-c -*- */
/**
 * 検波器
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 */
#include <stdio.h>
#include "../lib/io.h"
#include "../lib/signal_spec.h"

/**
 * 検波メイン
 *
 * @param stream_in  入力ストリーム
 * @param stream_out 出力ストリーム
 */
void detect_main ( FILE *stream_in, FILE *stream_out )
{
	int sg_prev = 0;
	double t_prev = 0;
	sample_t s_in, s_out;
	double dt;

	s_out.a = A_NONE;
	while ( !feof ( stream_in ) ) {
		if ( sample_read ( &s_in, 1, stream_in ) < 1 ) {
			break;
		}
		dt = s_in.t - t_prev;
		if ( ( sg_prev <= 0 ) && ( s_in.a > 0 ) ) {
			/* マイナス -> プラスのゼロクロス点を検出した */
			t_prev = s_in.t;

			if ( (T_MARK_L <= dt) && (dt <= T_MARK_H) ) {
				s_out.a = A_MARK;
			} else if ( (T_SPACE_L <= dt) && (dt <= T_SPACE_H) ) {
				s_out.a = A_SPACE;
			} else {
				s_out.a = A_NONE;
			}
		} else if ( T_SPACE_H < dt ) {
			s_out.a = A_NONE;
		}

		/* サンプルの符号を保存 */
		if ( s_in.a < 0 ) {
			sg_prev = -1;
		} else if ( s_in.a == 0 ) {
			sg_prev = 0;
		} else {
			sg_prev = 1;
		}
		s_out.t = s_in.t;
		sample_write ( &s_out, 1, stream_out );
		fflush ( stream_out );
	}
}



int main ( int argc, char *argv[] )
{
	detect_main ( stdin, stdout );
	return 0;
}
