/* -*- linux-c -*- */
/**
 * 緊急警報放送　符号テーブル
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 */

#include "codetable.h"
#include <stdio.h>

/**
 * 符号表
 *
 * 無線設備規則第九条の三第五号の規定に基づく緊急警報信号の構成　より
 *
 * http://www.tele.soumu.go.jp/horei/reiki_honbun/a721030001.html
 */
/* 前置符号 */
char code_zenchi_12[] =  {1,1,0,0, ENDCODE};
char code_zenchi_end[] = {0,0,1,1, ENDCODE};

/* 固定符号 */
char code_kotei_1end[] = {0,0,0,0, 1,1,1,0, 0,1,1,0, 1,1,0,1, ENDCODE};
char code_kotei_2[]    = {1,1,1,1, 0,0,0,1, 1,0,0,1, 0,0,1,0, ENDCODE};

/* 地域区分符号の開始部 */
char code_chiiki_pre_12[]  = {1,0, ENDCODE};
char code_chiiki_pre_end[] = {0,1, ENDCODE};
/* 地域区分符号の末尾部 */
char code_chiiki_post_12[]  = {0,0, ENDCODE};
char code_chiiki_post_end[] = {1,1, ENDCODE};

/* 月日区分符号の開始部 */
char code_tsukihi_pre_12[]  = {0,1,0, ENDCODE};
char code_tsukihi_pre_end[] = {1,0,0, ENDCODE};
/* 月日区分符号の日符号と月符号の間のビット */
char code_tsukihi_mid_odd[]  = {0, ENDCODE};
char code_tsukihi_mid_even[] = {1, ENDCODE};
/* 月日区分符号の末尾部 */
char code_tsukihi_post_12[]  = {0,0, ENDCODE};
char code_tsukihi_post_end[] = {1,1, ENDCODE};

/* 年時区分符号の開始部 */
char code_nenzi_pre_12[]  = {0,1,1, ENDCODE};
char code_nenzi_pre_end[] = {1,0,1, ENDCODE};
/* 年時区分符号の時符号と年符号の間のビット */
char code_nenzi_mid_odd[]  = {0, ENDCODE};
char code_nenzi_mid_even[] = {1, ENDCODE};
/* 年次区分符号の末尾部 */
char code_nenzi_post_12[]  = {0,0, ENDCODE};
char code_nenzi_post_end[] = {1,1, ENDCODE};


/* 地域符号 */
chiiki_code_t chiiki_codeset[] = {
	/* 地域共通符号 */
	{ "地域共通符号", {0,0,1,1, 0,1,0,0, 1,1,0,1, ENDCODE}, },

	/* 広域符号 */
	{ "関東広域圏",   {0,1,0,1, 1,0,1,0, 0,1,0,1, ENDCODE}, },
	{ "中京広域圏",   {0,1,1,1, 0,0,1,0, 1,0,1,0, ENDCODE}, },
	{ "近畿広域圏",   {1,0,0,0, 1,1,0,1, 0,1,0,1, ENDCODE}, },
	{ "鳥取・島根圏", {0,1,1,0, 1,0,0,1, 1,0,0,1, ENDCODE}, },
	{ "岡山・香川圏", {0,1,0,1, 0,1,0,1, 0,0,1,1, ENDCODE}, },

	/* 県域符号 */
	{ "北海道",   {0,0,0,1, 0,1,1,0, 1,0,1,1, ENDCODE}, },
	{ "青森県",   {0,1,0,0, 0,1,1,0, 0,1,1,1, ENDCODE}, },
	{ "岩手県",   {0,1,0,1, 1,1,0,1, 0,1,0,0, ENDCODE}, },
	{ "宮城県",   {0,1,1,1, 0,1,0,1, 1,0,0,0, ENDCODE}, },
	{ "秋田県",   {1,0,1,0, 1,1,0,0, 0,1,1,0, ENDCODE}, },
	{ "山形県",   {1,1,1,0, 0,1,0,0, 1,1,0,0, ENDCODE}, },
	{ "福島県",   {0,0,0,1, 1,0,1,0, 1,1,1,0, ENDCODE}, },
	{ "茨城県",   {1,1,0,0, 0,1,1,0, 1,0,0,1, ENDCODE}, },
	{ "栃木県",   {1,1,1,0, 0,0,1,1, 1,0,0,0, ENDCODE}, },
	{ "群馬県",   {1,0,0,1, 1,0,0,0, 1,0,1,1, ENDCODE}, },
	{ "埼玉県",   {0,1,1,0, 0,1,0,0, 1,0,1,1, ENDCODE}, },
	{ "千葉県",   {0,0,0,1, 1,1,0,0, 0,1,1,1, ENDCODE}, },
	{ "東京都",   {1,0,1,0, 1,0,1,0, 1,1,0,0, ENDCODE}, },
	{ "神奈川県", {0,1,0,1, 0,1,1,0, 1,1,0,0, ENDCODE}, },
	{ "新潟県",   {0,1,0,0, 1,1,0,0, 1,1,1,0, ENDCODE}, },
	{ "富山県",   {0,1,0,1, 0,0,1,1, 1,0,0,1, ENDCODE}, },
	{ "石川県",   {0,1,1,0, 1,0,1,0, 0,1,1,0, ENDCODE}, },
	{ "福井県",   {1,0,0,1, 0,0,1,0, 1,1,0,1, ENDCODE}, },
	{ "山梨県",   {1,1,0,1, 0,1,0,0, 1,0,1,0, ENDCODE}, },
	{ "長野県",   {1,0,0,1, 1,1,0,1, 0,0,1,0, ENDCODE}, },
	{ "岐阜県",   {1,0,1,0, 0,1,1,0, 0,1,0,1, ENDCODE}, },
	{ "静岡県",   {1,0,1,0, 0,1,0,1, 1,0,1,0, ENDCODE}, },
	{ "愛知県",   {1,0,0,1, 0,1,1,0, 0,1,1,0, ENDCODE}, },
	{ "三重県",   {0,0,1,0, 1,1,0,1, 1,1,0,0, ENDCODE}, },
	{ "滋賀県",   {1,1,0,0, 1,1,1,0, 0,1,0,0, ENDCODE}, },
	{ "京都府",   {0,1,0,1, 1,0,0,1, 1,0,1,0, ENDCODE}, },
	{ "大阪府",   {1,1,0,0, 1,0,1,1, 0,0,1,0, ENDCODE}, },
	{ "兵庫県",   {0,1,1,0, 0,1,1,1, 0,1,0,0, ENDCODE}, },
	{ "奈良県",   {1,0,1,0, 1,0,0,1, 0,0,1,1, ENDCODE}, },
	{ "和歌山県", {0,0,1,1, 1,0,0,1, 0,1,1,0, ENDCODE}, },
	{ "鳥取県",   {1,1,0,1, 0,0,1,0, 0,0,1,1, ENDCODE}, },
	{ "島根県",   {0,0,1,1, 0,0,0,1, 1,0,1,1, ENDCODE}, },
	{ "岡山県",   {0,0,1,0, 1,0,1,1, 0,1,0,1, ENDCODE}, },
	{ "広島県",   {1,0,1,1, 0,0,1,1, 0,0,0,1, ENDCODE}, },
	{ "山口県",   {1,0,1,1, 1,0,0,1, 1,0,0,0, ENDCODE}, },
	{ "徳島県",   {1,1,1,0, 0,1,1,0, 0,0,1,0, ENDCODE}, },
	{ "香川県",   {1,0,0,1, 1,0,1,1, 0,1,0,0, ENDCODE}, },
	{ "愛媛県",   {0,0,0,1, 1,0,0,1, 1,1,0,1, ENDCODE}, },
	{ "高知県",   {0,0,1,0, 1,1,1,0, 0,0,1,1, ENDCODE}, },
	{ "福岡県",   {0,1,1,0, 0,0,1,0, 1,1,0,1, ENDCODE}, },
	{ "佐賀県",   {1,0,0,1, 0,1,0,1, 1,0,0,1, ENDCODE}, },
	{ "長崎県",   {1,0,1,0, 0,0,1,0, 1,0,1,1, ENDCODE}, },
	{ "熊本県",   {1,0,0,0, 1,0,1,0, 0,1,1,1, ENDCODE}, },
	{ "大分県",   {1,1,0,0, 1,0,0,0, 1,1,0,1, ENDCODE}, },
	{ "宮崎県",   {1,1,0,1, 0,0,0,1, 1,1,0,0, ENDCODE}, },
	{ "鹿児島県", {1,1,0,1, 0,1,0,0, 0,1,0,1, ENDCODE}, },
	{ "沖縄県",   {0,0,1,1, 0,1,1,1, 0,0,1,0, ENDCODE}, },
	{ NULL, {}, }, /* 番人 */
};

/* 日符号表 */
hi_code_t hi_codeset[] = {
	{  1, {1,0,0,0,0, ENDCODE}, },
	{  2, {0,1,0,0,0, ENDCODE}, },
	{  3, {1,1,0,0,0, ENDCODE}, },
	{  4, {0,0,1,0,0, ENDCODE}, },
	{  5, {1,0,1,0,0, ENDCODE}, },
	{  6, {0,1,1,0,0, ENDCODE}, },
	{  7, {1,1,1,0,0, ENDCODE}, },
	{  8, {0,0,0,1,0, ENDCODE}, },
	{  9, {1,0,0,1,0, ENDCODE}, },
	{ 10, {0,1,0,1,0, ENDCODE}, },
	{ 11, {1,1,0,1,0, ENDCODE}, },
	{ 12, {0,0,1,1,0, ENDCODE}, },
	{ 13, {1,0,1,1,0, ENDCODE}, },
	{ 14, {0,1,1,1,0, ENDCODE}, },
	{ 15, {1,1,1,1,0, ENDCODE}, },
	{ 16, {0,0,0,0,1, ENDCODE}, },
	{ 17, {1,0,0,0,1, ENDCODE}, },
	{ 18, {0,1,0,0,1, ENDCODE}, },
	{ 19, {1,1,0,0,1, ENDCODE}, },
	{ 20, {0,0,1,0,1, ENDCODE}, },
	{ 21, {1,0,1,0,1, ENDCODE}, },
	{ 22, {0,1,1,0,1, ENDCODE}, },
	{ 23, {1,1,1,0,1, ENDCODE}, },
	{ 24, {0,0,0,1,1, ENDCODE}, },
	{ 25, {1,0,0,1,1, ENDCODE}, },
	{ 26, {0,1,0,1,1, ENDCODE}, },
	{ 27, {1,1,0,1,1, ENDCODE}, },
	{ 28, {0,0,1,1,1, ENDCODE}, },
	{ 29, {1,0,1,1,1, ENDCODE}, },
	{ 30, {0,1,1,1,1, ENDCODE}, },
	{ 31, {1,1,1,1,1, ENDCODE}, },
	{ -1, {}, }, /* 番人 */
};

/* 月符号表 */
tsuki_code_t tsuki_codeset[] = {
	{  1, {1,0,0,0,1, ENDCODE}, },
	{  2, {0,1,0,0,1, ENDCODE}, },
	{  3, {1,1,0,0,1, ENDCODE}, },
	{  4, {0,0,1,0,1, ENDCODE}, },
	{  5, {1,0,1,0,1, ENDCODE}, },
	{  6, {0,1,1,0,1, ENDCODE}, },
	{  7, {1,1,1,0,1, ENDCODE}, },
	{  8, {0,0,0,1,1, ENDCODE}, },
	{  9, {1,0,0,1,1, ENDCODE}, },
	{ 10, {0,1,0,1,1, ENDCODE}, },
	{ 11, {1,1,0,1,1, ENDCODE}, },
	{ 12, {0,0,1,1,1, ENDCODE}, },
	{ -1, {},}, /* 番人 */
};

/* 時符号表 */
zi_code_t zi_codeset[] = {
	{  0, {0,0,0,1,1, ENDCODE}, },
	{  1, {1,0,0,1,1, ENDCODE}, },
	{  2, {0,1,0,1,1, ENDCODE}, },
	{  3, {1,1,0,1,1, ENDCODE}, },
	{  4, {0,0,1,1,1, ENDCODE}, },
	{  5, {1,0,1,1,1, ENDCODE}, },
	{  6, {0,1,1,1,1, ENDCODE}, },
	{  7, {1,1,1,1,1, ENDCODE}, },
	{  8, {0,0,0,0,1, ENDCODE}, },
	{  9, {1,0,0,0,1, ENDCODE}, },
	{ 10, {0,1,0,0,1, ENDCODE}, },
	{ 11, {1,1,0,0,1, ENDCODE}, },
	{ 12, {0,0,1,0,1, ENDCODE}, },
	{ 13, {1,0,1,0,1, ENDCODE}, },
	{ 14, {0,1,1,0,1, ENDCODE}, },
	{ 15, {1,1,1,0,1, ENDCODE}, },
	{ 16, {0,0,0,1,0, ENDCODE}, },
	{ 17, {1,0,0,1,0, ENDCODE}, },
	{ 18, {0,1,0,1,0, ENDCODE}, },
	{ 19, {1,1,0,1,0, ENDCODE}, },
	{ 20, {0,0,1,1,0, ENDCODE}, },
	{ 21, {1,0,1,1,0, ENDCODE}, },
	{ 22, {0,1,1,1,0, ENDCODE}, },
	{ 23, {1,1,1,1,0, ENDCODE}, },
	{ -1, {}, }, /* 番人 */
};

/* 年符号表．年号は昭和 */
nen_code_t nen_codeset[] = {
	{ 60, {1,0,1,0,1, ENDCODE}, },
	{ 61, {0,1,1,0,1, ENDCODE}, },
	{ 62, {1,1,1,0,1, ENDCODE}, },
	{ 63, {0,0,0,1,1, ENDCODE}, },
	{ 64, {1,0,0,1,1, ENDCODE}, },
	{ 65, {0,1,0,1,1, ENDCODE}, },
	{ 66, {1,0,0,0,1, ENDCODE}, },
	{ 67, {0,1,0,0,1, ENDCODE}, },
	{ 68, {1,1,0,0,1, ENDCODE}, },
	{ 69, {0,0,1,0,1, ENDCODE}, },
	{ -1, {}, }, /* 番人 */
};
