/* -*- linux-c -*- */
/**
 * 符号テーブル　型定義
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 */
#ifndef __CODETABLE_H__
#define __CODETABLE_H__

#define ENDCODE (99) /* 符号の終わりを示す番人 */

/* 前置符号 */
extern char code_zenchi_12[];
extern char code_zenchi_end[];

/* 固定符号 */
extern char code_kotei_1end[];
extern char code_kotei_2[];

/* 地域区分符号の開始部 */
extern char code_chiiki_pre_12[];
extern char code_chiiki_pre_end[];
/* 地域区分符号の末尾部 */
extern char code_chiiki_post_12[];
extern char code_chiiki_post_end[];

/* 月日区分符号の開始部 */
extern char code_tsukihi_pre_12[];
extern char code_tsukihi_pre_end[];
/* 月日区分符号の日符号と月符号の間のビット */
extern char code_tsukihi_mid_odd[];
extern char code_tsukihi_mid_even[];
/* 月日区分符号の末尾部 */
extern char code_tsukihi_post_12[];
extern char code_tsukihi_post_end[];

/* 年時区分符号の開始部 */
extern char code_nenzi_pre_12[];
extern char code_nenzi_pre_end[];
/* 年時区分符号の時符号と年符号の間のビット */
extern char code_nenzi_mid_odd[];
extern char code_nenzi_mid_even[];
/* 年次区分符号の末尾部 */
extern char code_nenzi_post_12[];
extern char code_nenzi_post_end[];

/* 地域符号 */
typedef struct {
	char *name; /**< 地域名 */
	char code[13]; /**< 符号列 */
} chiiki_code_t;

/*
 * 地域符号（地域共通符号・広域符号・県域符号）の表
 * 末尾要素は { NULL, {} }
 */
extern chiiki_code_t chiiki_codeset[];

/* 日符号 */
typedef struct {
	int day;    /**< 日 */
	char code[6]; /**< 符号 */
} hi_code_t;

/*
 * 日符号表
 * 末尾は要素は { -1, NULL } 
 */
extern hi_code_t hi_codeset[];

/* 月符号 */
typedef struct {
	int month;  /**< 月 */
	char code[6]; /**< 符号 */
} tsuki_code_t;

/*
 * 月符号表
 * 末尾要素は { -1, NULL } 
 */
extern tsuki_code_t tsuki_codeset[];

/* 時符号 */
typedef struct {
	int hour;   /**< 時 */
	char code[6]; /**< 符号 */
} zi_code_t;

/*
 * 時符号表
 * 末尾要素は { -1, NULL } 
 */
extern zi_code_t zi_codeset[];

/* 年符号 */
typedef struct {
	int year;    /**< 年．昭和表記 */
	char code[6];  /**< 符号 */
} nen_code_t;

/*
 * 年符号表
 * 末尾要素は { -1, NULL } 
 */
extern nen_code_t nen_codeset[];

#endif /* __CODETABLE_H__ */
