/* -*- linux-c -*- */
/*
 * sox dat 形式入出力
 * 
 * (c) 2013-2014 Kunihiko Imai <bak@d2.dion.ne.jp>
 * All rights reserved.
 */
#include <stdio.h>
#include "io.h"

/**
 * sox の dat 形式ストリームの読み込み
 *
 * 先頭の ; で始まる行は読み飛ばす
 *
 * @param[out] buf    サンプル保持バッファ
 * @param[in]  len    サンプル保持バッファ長．サンプル数．
 * @param[in]  stream 入力ストリーム
 *
 * @return 読み込んだサンプル数
 */
int sample_read(sample_t *buf, int len, FILE *stream)
{
	char lbuf[1024];
	int i;

	for ( i = 0; i < len; ++i ) {
		do {
			if ( fgets ( lbuf, sizeof(lbuf), stream) == NULL ) {
				/* EOF */
				break;
			}
		} while ( lbuf[0] == ';' ); /* コメント行 */
		sscanf ( lbuf, "%lf %lf", &buf[i].t, &buf[i].a );
	}
	return i;
}


/**
 * sox の dat 形式ストリームの書き込み
 *
 * 先頭の ; で始まる行はケアしない
 *
 * @param[in]  buf    サンプル保持バッファ
 * @param[in]  len    サンプル保持バッファ長．サンプル数．
 * @param[out] stream 入力ストリーム
 *
 * @return 書き出したサンプル数
 */
int sample_write(sample_t *buf, int len, FILE *stream)
{
	int i;

	for ( i = 0; i < len; ++i ) {
		if ( fprintf ( stream, "%e\t%e\n", buf[i].t, buf[i].a ) <= 0 ) {
			break;
		}
	}
	return i;
}
